unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, ExtCtrls, cbDSMixer, StdCtrls, ComCtrls, contnrs;

const
  frameLength = 50;// in miliseconds

type
  TAnalysis = class
  private
    FChannelCount,
    FSamplesPerPixel,
    FSampleCount,
    FGraphWidth: Integer;
    FAddOn: Byte;
    FGraphData: array [0..1] of array of array of byte;
    FVUData: array [0..1] of Single;
    FChannel: TcbDSMixerChannel;

  public
    constructor Create(Channel: TcbDSMixerChannel; SampleCount,
      GraphWidth: Integer);
    procedure CalcGraph;
    procedure Clear;
  end;

  TForm1 = class(TForm)
    grid: TStringGrid;
    paintboxLeft: TPaintBox;
    Button1: TButton;
    OpenDialog1: TOpenDialog;
    Timer1: TTimer;
    paintboxRight: TPaintBox;
    pbarLeft: TProgressBar;
    pbarRight: TProgressBar;
    Button2: TButton;
    procedure gridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure Button1Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure paintboxLeftPaint(Sender: TObject);
    procedure paintboxRightPaint(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
    FWB: TcbDSMixerChannel;
    FAnalysis: TAnalysis;
    FDSFXManager: TDSFXManager;
    samples:integer;
    procedure OnEndPlay(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  GRAPH_WIDTH :integer = 110;
  smallrect:TRect=(left:0;top:0;right:100;bottom:100);
  bigrect:TRect=(left:0;top:0;right:100;bottom:100);

implementation

{$R *.dfm}

constructor TAnalysis.Create(Channel: TcbDSMixerChannel; SampleCount,
  GraphWidth: Integer);
begin
  inherited Create;

  FChannel := Channel;
  FGraphWidth := GraphWidth;
  FSampleCount := SampleCount;

  if (FChannel.FileNumChannels > 1) then
    FChannelCount := 2
  else
    FChannelCount := 1;

  FSamplesPerPixel := SampleCount div GRAPH_WIDTH;
  if (FSamplesPerPixel = 0) then
    FSamplesPerPixel := 1;

  if (FChannel.FileBitsPerSample = 8) then
    FAddOn := 0
  else
    FAddOn := 128;

  SetLength(FGraphData[0], FGraphWidth, FChannelCount);
  SetLength(FGraphData[1], FGraphWidth, FChannelCount);
end;

procedure TAnalysis.CalcGraph;
var
  GraphPoint,
  SamplePoint: Integer;
  Size1,
  Size2,
  CntCh: DWord;
  PointUpper,
  PointLower,
  MinValue,
  MaxValue: Byte;
  Audio1Ptr,
  Audio2Ptr: Pointer;

  procedure DoBufCalc(AudioPtr: Pointer; SamplesCount, Channel: Integer);
  var
    V: Byte;
  begin
    while (GraphPoint < FGraphWidth) do
    begin
      while (SamplePoint <= FSamplesPerPixel) do
      begin
        if (SamplesCount = 0) then
          exit;

        v := ShortInt(AudioPtr^) + FAddOn;
        if v > PointUpper then PointUpper := v;
        if v < PointLower then PointLower := v;
        Inc(SamplePoint);
        Inc(PByte(AudioPtr), FChannel.FileBlockAlign);
        Dec(SamplesCount);
      end;

      if (PointUpper > MaxValue) then
        MaxValue := PointUpper;
      if (PointLower < MinValue) then
        MinValue := PointLower;

      FGraphData[0, GraphPoint, Channel] := PointUpper;
      FGraphData[1, GraphPoint, Channel] := PointLower;
      PointUpper := 0;
      PointLower := 255;
      Inc(GraphPoint);
      SamplePoint := 1;
    end;
  end;

begin
  FChannel.Lock(0, FSampleCount*Integer(FChannel.FileBlockAlign),
    Audio1Ptr, Size1, Audio2Ptr, Size2);
  try
    for CntCh := 0 to FChannelCount-1 do
    begin
      MinValue := 255;
      MaxValue := 0;
      PointUpper := 0;
      PointLower := 255;
      GraphPoint := 0;
      SamplePoint := 1;
      if (Size1 > 0) then
        DoBufCalc(Pointer(PChar(Audio1Ptr) + FChannel.FileBlockAlign - 1 -
          (FChannel.FileBitsPerSample div 8) * CntCh),
          Size1 div FChannel.FileBlockAlign,
          CntCh);
      if (Size2 > 0) then
        DoBufCalc(Pointer(PChar(Audio2Ptr) + FChannel.FileBlockAlign - 1 -
          (FChannel.FileBitsPerSample div 8) * CntCh),
          Size2 div FChannel.FileBlockAlign,
          CntCh);

      if Abs(MinValue-128) > Abs(MaxValue-128) then
        FVUData[CntCh] := Sqrt(Abs(MinValue-128) / 128)
      else
        FVUData[CntCh] := Sqrt(Abs(MaxValue-128) / 128);
    end;
  finally
    FChannel.Unlock;
  end;
end;

procedure TAnalysis.Clear;
var
  Cnt,
  CntC: Integer;
begin
  for Cnt := 0 to FGraphWidth-1 do
    for CntC := 0 to FChannelCount-1 do
    begin
      FGraphData[0, Cnt, CntC] := 128;
      FGraphData[1, Cnt, CntC] := 128;
    end;
end;
//------------------------
procedure TForm1.gridDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
begin
  if ARow=0 then exit;
  if ACol < 2 then
  begin
    //Setting canvas properties and erasing old cellcontent:
    with grid.Canvas do
    begin
      if ARow>samples then
      begin
        Brush.Color := clWhite;
        FillRect(rect);
      end                      else{}
        CopyRect(rect, TBitmap(grid.Objects[ACol, ARow]).Canvas, smallrect);
    end;
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
var i:integer;
begin
  if OpenDialog1.Execute then
  begin
    FWB.FileName := '';
    FreeAndNil(FAnalysis);

    FWB.FileName := OpenDialog1.FileName;
    FAnalysis := TAnalysis.Create(FWB, (FWB.FileSamplesPerSec *
      frameLength) div 1000, GRAPH_WIDTH);
    fwb.Play;
    fwb.Stop;
    FDSFXManager.DSBuffer:=fwb.DSBuffer;
    FDSFXManager.LoadDefaultParameters;
    for i:=1 to 10 do
    begin
      sleep(100);
      application.processmessages;
    end;
    fwb.Play;
    timer1.enabled:=true;

    grid.cols[0].Clear;
    grid.cols[1].Clear;
    grid.cols[2].Clear;
    grid.cols[0].add('left');
    grid.cols[1].add('right');
    grid.cols[2].add('sample');
  end;
end;

function createBitmapFrom(pb:TPaintBox):TBitmap;
var b:tbitmap;
begin
  result:=TBitmap.Create;
  b:=TBitmap.Create;
  b.Height:=bigrect.Bottom-bigrect.Top;
  b.Width:=bigrect.Right-bigrect.Left+1;
  b.canvas.CopyRect(bigrect,pb.Canvas,bigrect);
  result.height:=smallrect.bottom-smallrect.top+1;
  result.width:=smallrect.right-smallrect.left+1;
  result.Canvas.StretchDraw(smallrect,b);
  b.free;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  if not(FWB.Status = sPlaying) then
    exit;

  inc(samples);
  
  FAnalysis.CalcGraph;

  paintboxLeft.Invalidate;
  paintboxRight.Invalidate;

  grid.Invalidate;

  pbarLeft.Position := Trunc(FAnalysis.FVUData[0]*100);
  pbarRight.Position := Trunc(FAnalysis.FVUData[1]*100);

  grid.Cols[0].AddObject('-',createBitmapFrom(paintboxLeft));
  grid.Cols[1].AddObject('-',createBitmapFrom(paintboxRight));
  grid.Cols[2].Add('Sample '+inttostr(samples));
  grid.RowCount:=grid.RowCount+1;
end;

procedure TForm1.OnEndPlay(Sender: TObject);
begin
  Timer1.Enabled := false;
  FAnalysis.Clear;
  paintboxLeft.Invalidate;
  paintboxRight.Invalidate;
end;

procedure TForm1.FormCreate(Sender: TObject);
var i:integer;
begin
  samples:=0;
  grid.RowCount:=2;
  for i:=1 to 3 do
    grid.Cols[i-1].Clear;

  GRAPH_WIDTH:=paintboxLeft.Width;

  smallrect.Right:=grid.DefaultColWidth;
  smallrect.Bottom:=grid.DefaultRowHeight;

  bigrect.Right:=paintboxLeft.Width;
  bigrect.Bottom:=paintboxLeft.Height;
  
  timer1.Interval:=frameLength;
  if not Assigned(FWB) then
  begin
    FWB := TcbDSMixerChannel.Create(TcbDSMixer.Create(self));

//    FWB.OnEndFade := OnEndFade;
    FWB.OnEndPlay := OnEndPlay;
  end;

  FDSFXManager:=TDSFXManager.Create(Self);
  paintboxLeft.Canvas.Brush.Style:=bsclear;
  paintboxright.Canvas.Brush.Style:=bsclear;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var i,j:integer;
begin
  timer1.Enabled := False;
  for j:=0 to 1 do
    for i:=1 to grid.Cols[j].Count do
      grid.Cols[j].Objects[i-1].Free;
  FreeAndNil(FAnalysis);
  FreeAndNil(FWB);
  FreeAndNil(FDSFXManager);
end;

procedure TForm1.paintboxLeftPaint(Sender: TObject);
var
  Cnt: Integer;
begin
  if Assigned(FAnalysis) then
    for Cnt := 0 to FAnalysis.FGraphWidth-1 do
    begin
      paintboxleft.Canvas.Pen.Color:=clGray;
      paintboxLeft.Canvas.MoveTo(Cnt, (255-FAnalysis.FGraphData[0, Cnt, 0]) shr 2 );
      paintboxLeft.Canvas.LineTo(Cnt, (255-FAnalysis.FGraphData[1, Cnt, 0]) shr 2 );
      paintboxleft.Canvas.Pen.Color:=clwhite;
      paintboxLeft.Canvas.MoveTo(Cnt-1, (255-FAnalysis.FGraphData[0, Cnt, 0]) shr 2-1 );
      paintboxLeft.Canvas.LineTo(Cnt-1, (255-FAnalysis.FGraphData[1, Cnt, 0]) shr 2-1 );
      paintboxleft.Canvas.Pen.Color:=clblack;
      paintboxLeft.Canvas.MoveTo(Cnt+1, (255-FAnalysis.FGraphData[0, Cnt, 0]) shr 2 +1);
      paintboxLeft.Canvas.LineTo(Cnt+1, (255-FAnalysis.FGraphData[1, Cnt, 0]) shr 2 +1);
    end;
end;

procedure TForm1.paintboxRightPaint(Sender: TObject);
var
  Cnt: Integer;
begin
  if Assigned(FAnalysis) and (FAnalysis.FChannelCount > 1) then
    for Cnt := 0 to FAnalysis.FGraphWidth-1 do
    begin
      paintboxRight.Canvas.Pen.Color:=clblue;
      paintboxRight.Canvas.MoveTo(Cnt-1, (255-FAnalysis.FGraphData[0, Cnt, 1]) shr 2 );
      paintboxRight.Canvas.LineTo(Cnt-1, (255-FAnalysis.FGraphData [1, Cnt, 1]) shr 2 );
      paintboxRight.Canvas.Pen.Color:=clblack;
      paintboxRight.Canvas.MoveTo(Cnt, (255-FAnalysis.FGraphData[0, Cnt, 1]) shr 2 +1);
      paintboxRight.Canvas.LineTo(Cnt, (255-FAnalysis.FGraphData [1, Cnt, 1]) shr 2 +1);
    end;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  fwb.Stop;
  if samples>1 then
    grid.RowCount:=grid.RowCount-1;
end;

end.
